@@ -30,19 +30,33 @@
  * with types changed to match glib types, since those are defined for us.
  * However, upstream FAAD is distributed with a broken header file that defined
  * these wrongly (in a way which was broken on 64 bit systems).
+ *
  * Upstream CVS still has the bug, but has also renamed all the public symbols
- * for Better Corporate Branding (or whatever), so we're screwed there.
+ * for Better Corporate Branding (or whatever), so we need to take that
+ * (FAAD_IS_NEAAC) into account as well.
  *
  * We must call them using these definitions. Most distributions now have the
  * corrected header file (they distribute a patch along with the source), 
- * but not all, hence this Truly Evil Hack. This hack will need updating if
- * upstream ever releases something with the new API.
+ * but not all, hence this Truly Evil Hack.
+ *
+ * Note: The prototypes don't need to be defined conditionaly, as the cpp will
+ * do that for us.
  */
+#ifdef FAAD_IS_NEAAC
+#define NeAACDecInit NeAACDecInit_no_definition
+#define NeAACDecInit2 NeAACDecInit2_no_definition
+#else
 #define faacDecInit faacDecInit_no_definition
 #define faacDecInit2 faacDecInit2_no_definition
+#endif
 #include "gstfaad.h"
+#ifdef FAAD_IS_NEAAC
+#undef NeAACDecInit
+#undef NeAACDecInit2
+#else
 #undef faacDecInit
 #undef faacDecInit2
+#endif
 
 extern long faacDecInit (faacDecHandle, guint8 *, guint32, guint32 *, guint8 *);
 extern int8_t faacDecInit2 (faacDecHandle, guint8 *, guint32,
@@ -309,7 +323,8 @@ gst_faad_setcaps (GstPad * pad, GstCaps * caps)
             &channels) < 0)
       goto init_failed;
 
-    GST_DEBUG_OBJECT (faad, "channels=%u, rate=%u", channels, samplerate);
+    GST_DEBUG_OBJECT (faad, "codec_data init: channels=%u, rate=%u", channels,
+        samplerate);
 
     /* not updating these here, so they are updated in the
      * chain function, and new caps are created etc. */
@@ -325,6 +340,7 @@ gst_faad_setcaps (GstPad * pad, GstCaps * caps)
   } else if ((value = gst_structure_get_value (str, "framed")) &&
       g_value_get_boolean (value) == TRUE) {
     faad->packetised = TRUE;
+    GST_DEBUG_OBJECT (faad, "we have packetized audio");
   } else {
     faad->init = FALSE;
   }
@@ -424,13 +440,27 @@ gst_faad_chanpos_from_gst (GstAudioChannelPosition * pos, guint num)
 */
 
 static GstAudioChannelPosition *
-gst_faad_chanpos_to_gst (guchar * fpos, guint num)
+gst_faad_chanpos_to_gst (GstFaad * faad, guchar * fpos, guint num,
+    gboolean * channel_map_failed)
 {
   GstAudioChannelPosition *pos = g_new (GstAudioChannelPosition, num);
   guint n;
   gboolean unknown_channel = FALSE;
 
+  *channel_map_failed = FALSE;
+
+  /* special handling for the common cases for mono and stereo */
+  if (num == 1 && fpos[0] == FRONT_CHANNEL_CENTER) {
+    GST_DEBUG_OBJECT (faad, "mono common case; won't set channel positions");
+    return NULL;
+  } else if (num == 2 && fpos[0] == FRONT_CHANNEL_LEFT
+      && fpos[1] == FRONT_CHANNEL_RIGHT) {
+    GST_DEBUG_OBJECT (faad, "stereo common case; won't set channel positions");
+    return NULL;
+  }
+
   for (n = 0; n < num; n++) {
+    GST_DEBUG_OBJECT (faad, "faad channel %d as %d", n, fpos[n]);
     switch (fpos[n]) {
       case FRONT_CHANNEL_LEFT:
         pos[n] = GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT;
@@ -464,28 +494,29 @@ gst_faad_chanpos_to_gst (guchar * fpos, guint num)
         pos[n] = GST_AUDIO_CHANNEL_POSITION_LFE;
         break;
       default:
+        GST_DEBUG_OBJECT (faad, "unknown channel %d at %d", fpos[n], n);
         unknown_channel = TRUE;
         break;
     }
   }
   if (unknown_channel) {
+    g_free (pos);
+    pos = NULL;
     switch (num) {
       case 1:{
-        GST_DEBUG ("FAAD reports unknown 1 channel mapping. Forcing to mono");
-        pos[0] = GST_AUDIO_CHANNEL_POSITION_FRONT_MONO;
+        GST_DEBUG_OBJECT (faad,
+            "FAAD reports unknown 1 channel mapping. Forcing to mono");
         break;
       }
       case 2:{
-        GST_DEBUG ("FAAD reports unknown 2 channel mapping. Forcing to stereo");
-        pos[0] = GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT;
-        pos[1] = GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT;
+        GST_DEBUG_OBJECT (faad,
+            "FAAD reports unknown 2 channel mapping. Forcing to stereo");
         break;
       }
       default:{
         GST_WARNING ("Unsupported FAAD channel position 0x%x encountered",
             fpos[n]);
-        g_free (pos);
-        pos = NULL;
+        *channel_map_failed = TRUE;
         break;
       }
     }
@@ -505,12 +536,13 @@ gst_faad_srcgetcaps (GstPad * pad)
   if (!supported_positions) {
     guchar *supported_fpos = g_new0 (guchar, num_supported_positions);
     gint n;
+    gboolean map_failed;
 
     for (n = 0; n < num_supported_positions; n++) {
       supported_fpos[n] = n + FRONT_CHANNEL_CENTER;
     }
-    supported_positions = gst_faad_chanpos_to_gst (supported_fpos,
-        num_supported_positions);
+    supported_positions = gst_faad_chanpos_to_gst (faad, supported_fpos,
+        num_supported_positions, &map_failed);
     g_free (supported_fpos);
   }
 
@@ -575,15 +607,18 @@ gst_faad_srcgetcaps (GstPad * pad)
         /* put channel information here */
         if (faad->channel_positions) {
           GstAudioChannelPosition *pos;
+          gboolean map_failed;
 
-          pos = gst_faad_chanpos_to_gst (faad->channel_positions,
-              faad->channels);
-          if (!pos) {
+          pos = gst_faad_chanpos_to_gst (faad, faad->channel_positions,
+              faad->channels, &map_failed);
+          if (map_failed) {
             gst_structure_free (str);
             continue;
           }
-          gst_audio_set_channel_positions (str, pos);
-          g_free (pos);
+          if (pos) {
+            gst_audio_set_channel_positions (str, pos);
+            g_free (pos);
+          }
         } else {
           gst_audio_set_structure_channel_positions_list (str,
               supported_positions, num_supported_positions);
@@ -997,6 +1032,7 @@ gst_faad_update_caps (GstFaad * faad, faacDecFrameInfo * info)
 {
   GstAudioChannelPosition *pos;
   gboolean ret;
+  gboolean channel_map_failed;
   GstCaps *caps;
 
   /* store new negotiation information */
@@ -1015,16 +1051,21 @@ gst_faad_update_caps (GstFaad * faad, faacDecFrameInfo * info)
 
   faad->bps = 16 / 8;
 
-  pos = gst_faad_chanpos_to_gst (faad->channel_positions, faad->channels);
-  if (!pos) {
+  channel_map_failed = FALSE;
+  pos =
+      gst_faad_chanpos_to_gst (faad, faad->channel_positions, faad->channels,
+      &channel_map_failed);
+  if (channel_map_failed) {
     GST_DEBUG_OBJECT (faad, "Could not map channel positions");
     gst_caps_unref (caps);
     return FALSE;
   }
-  gst_audio_set_channel_positions (gst_caps_get_structure (caps, 0), pos);
-  g_free (pos);
+  if (pos) {
+    gst_audio_set_channel_positions (gst_caps_get_structure (caps, 0), pos);
+    g_free (pos);
+  }
 
-  GST_DEBUG ("New output caps: %" GST_PTR_FORMAT, caps);
+  GST_DEBUG_OBJECT (faad, "New output caps: %" GST_PTR_FORMAT, caps);
 
   ret = gst_pad_set_caps (faad->srcpad, caps);
   gst_caps_unref (caps);
@@ -1118,56 +1159,6 @@ looks_like_valid_header (guint8 * input_data, guint input_size)
   return TRUE;
 }
 
-/*
-  clips buffer to currently configured segment. Returns FALSE if the buffer 
-  has to be dropped.
-*/
-
-static gboolean
-clip_outgoing_buffer (GstFaad * faad, GstBuffer * buffer)
-{
-  gint64 start, stop, cstart, cstop, diff;
-  gboolean res = TRUE;
-
-  if (faad->segment->format != GST_FORMAT_TIME)
-    goto beach;
-
-  start = GST_BUFFER_TIMESTAMP (buffer);
-  stop = start + GST_BUFFER_DURATION (buffer);
-
-  if (gst_segment_clip (faad->segment, GST_FORMAT_TIME,
-          start, stop, &cstart, &cstop)) {
-    diff = cstart - start;
-    if (diff > 0) {
-      GST_BUFFER_TIMESTAMP (buffer) = cstart;
-      GST_BUFFER_DURATION (buffer) -= diff;
-
-      /* time->frames->bytes */
-      diff =
-          faad->bps * faad->channels * GST_CLOCK_TIME_TO_FRAMES (diff,
-          faad->samplerate);
-      GST_BUFFER_DATA (buffer) += diff;
-      GST_BUFFER_SIZE (buffer) -= diff;
-    }
-    diff = cstop - stop;
-    if (diff > 0) {
-      GST_BUFFER_DURATION (buffer) -= diff;
-      /* time->frames->bytes */
-      diff =
-          faad->bps * faad->channels * GST_CLOCK_TIME_TO_FRAMES (diff,
-          faad->samplerate);
-      /* update size */
-      GST_BUFFER_SIZE (buffer) -= diff;
-    }
-  } else {
-    GST_DEBUG_OBJECT (faad, "buffer is outside configured segment");
-    res = FALSE;
-  }
-
-beach:
-  return res;
-}
-
 static GstFlowReturn
 gst_faad_chain (GstPad * pad, GstBuffer * buffer)
 {
@@ -1375,7 +1366,8 @@ gst_faad_chain (GstPad * pad, GstBuffer * buffer)
         faad->sum_dur_out += GST_BUFFER_DURATION (outbuf);
         GST_OBJECT_UNLOCK (faad);
 
-        if (clip_outgoing_buffer (faad, outbuf)) {
+        if ((outbuf = gst_audio_buffer_clip (outbuf, faad->segment,
+                    faad->samplerate, faad->bps * faad->channels))) {
           GST_LOG_OBJECT (faad,
               "pushing buffer, off=%" G_GUINT64_FORMAT ", ts=%" GST_TIME_FORMAT,
               GST_BUFFER_OFFSET (outbuf),
@@ -1383,8 +1375,7 @@ gst_faad_chain (GstPad * pad, GstBuffer * buffer)
           ret = gst_pad_push (faad->srcpad, outbuf);
           if (ret != GST_FLOW_OK)
             goto out;
-        } else
-          gst_buffer_unref (outbuf);
+        }
       }
     }
   }
@@ -1447,7 +1438,7 @@ gst_faad_open_decoder (GstFaad * faad)
 
   conf = faacDecGetCurrentConfiguration (faad->handle);
   conf->defObjectType = LC;
-  /* conf->dontUpSampleImplicitSBR = 1; */
+  conf->dontUpSampleImplicitSBR = 1;
   conf->outputFormat = FAAD_FMT_16BIT;
 
   if (faacDecSetConfiguration (faad->handle, conf) == 0) {
@@ -1479,7 +1470,7 @@ gst_faad_change_state (GstElement * element, GstStateChange transition)
         return GST_STATE_CHANGE_FAILURE;
       break;
     case GST_STATE_CHANGE_READY_TO_PAUSED:
-      gst_segment_init (faad->segment, GST_FORMAT_UNDEFINED);
+      gst_segment_init (faad->segment, GST_FORMAT_TIME);
       break;
     default:
       break;
