@@ -19,8 +19,9 @@
  */
 
 /**
- * SECTION:element-theoradec
- * @see_also: theoraenc, oggdemux
+ * SECTION:element-theoradecexp
+ * @short_description: decoder theora video using the experimental decoder
+ * @see_also: theoradec, theoraenc, oggdemux
  *
  * <refsect2>
  * <para>
@@ -48,7 +49,7 @@
 #include "theoradec.h"
 #include <gst/tag/tag.h>
 
-GST_DEBUG_CATEGORY (theoradecexp_debug);
+GST_DEBUG_CATEGORY_STATIC (theoradecexp_debug);
 #define GST_CAT_DEFAULT theoradecexp_debug
 
 static const GstElementDetails theora_dec_details =
@@ -58,15 +59,12 @@ GST_ELEMENT_DETAILS ("Theora video decoder",
     "Benjamin Otte <in7y118@public.uni-hamburg.de>, "
     "Wim Taymans <wim@fluendo.com>, " "Michael Smith <msmith@fluendo,com>");
 
-/* TODO: Support for other pixel formats (4:4:4) as supported by the
- * theoraexp codebase
- */
 static GstStaticPadTemplate theora_dec_src_factory =
 GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS ("video/x-raw-yuv, "
-        "format = (fourcc) { I420, YUY2 }, "
+        "format = (fourcc) { I420, YUY2, Y444 }, "
         "framerate = (fraction) [0/1, MAX], "
         "width = (int) [ 1, MAX ], " "height = (int) [ 1, MAX ]")
     );
@@ -144,15 +142,25 @@ gst_theoradec_init (GstTheoraExpDec * dec, GstTheoraExpDecClass * g_class)
 static void
 gst_theoradec_reset (GstTheoraExpDec * dec)
 {
+  GList *walk;
+
   dec->need_keyframe = TRUE;
   dec->last_timestamp = -1;
   dec->granulepos = -1;
+  dec->discont = TRUE;
+  dec->frame_nr = -1;
   gst_segment_init (&dec->segment, GST_FORMAT_TIME);
 
   GST_OBJECT_LOCK (dec);
   dec->proportion = 1.0;
   dec->earliest_time = -1;
   GST_OBJECT_UNLOCK (dec);
+
+  for (walk = dec->queued; walk; walk = g_list_next (walk)) {
+    gst_buffer_unref (GST_BUFFER_CAST (walk->data));
+  }
+  g_list_free (dec->queued);
+  dec->queued = NULL;
 }
 
 static gint64
@@ -173,6 +181,8 @@ theora_get_query_types (GstPad * pad)
 {
   static const GstQueryType theora_src_query_types[] = {
     GST_QUERY_POSITION,
+    GST_QUERY_DURATION,
+    GST_QUERY_CONVERT,
     0
   };
 
@@ -208,7 +218,7 @@ theora_dec_src_convert (GstPad * pad,
     return TRUE;
   }
 
-  dec = GST_THEORA_DEC (gst_pad_get_parent (pad));
+  dec = GST_THEORA_DEC_EXP (gst_pad_get_parent (pad));
 
   /* we need the info part before we can done something */
   if (!dec->have_header)
@@ -291,7 +301,7 @@ theora_dec_sink_convert (GstPad * pad,
     return TRUE;
   }
 
-  dec = GST_THEORA_DEC (gst_pad_get_parent (pad));
+  dec = GST_THEORA_DEC_EXP (gst_pad_get_parent (pad));
 
   /* we need the info part before we can done something */
   if (!dec->have_header)
@@ -357,7 +367,7 @@ theora_dec_src_query (GstPad * pad, GstQuery * query)
 
   gboolean res = FALSE;
 
-  dec = GST_THEORA_DEC (gst_pad_get_parent (pad));
+  dec = GST_THEORA_DEC_EXP (gst_pad_get_parent (pad));
 
   switch (GST_QUERY_TYPE (query)) {
     case GST_QUERY_POSITION:
@@ -383,7 +393,7 @@ theora_dec_src_query (GstPad * pad, GstQuery * query)
                   granulepos, &my_format, &time)))
         goto error;
 
-      time = (time - dec->segment.start) + dec->segment.time;
+      time = gst_segment_to_stream_time (&dec->segment, GST_FORMAT_TIME, time);
 
       GST_LOG_OBJECT (dec,
           "query %p: our time: %" GST_TIME_FORMAT, query, GST_TIME_ARGS (time));
@@ -400,10 +410,19 @@ theora_dec_src_query (GstPad * pad, GstQuery * query)
       break;
     }
     case GST_QUERY_DURATION:
+    {
       /* forward to peer for total */
-      if (!(res = gst_pad_query (GST_PAD_PEER (dec->sinkpad), query)))
+      GstPad *peer;
+
+      if (!(peer = gst_pad_get_peer (dec->sinkpad)))
+        goto error;
+
+      res = gst_pad_query (peer, query);
+      gst_object_unref (peer);
+      if (!res)
         goto error;
       break;
+    }
     case GST_QUERY_CONVERT:
     {
       GstFormat src_fmt, dest_fmt;
@@ -470,7 +489,7 @@ theora_dec_src_event (GstPad * pad, GstEvent * event)
   gboolean res = TRUE;
   GstTheoraExpDec *dec;
 
-  dec = GST_THEORA_DEC (gst_pad_get_parent (pad));
+  dec = GST_THEORA_DEC_EXP (gst_pad_get_parent (pad));
 
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_SEEK:
@@ -485,6 +504,7 @@ theora_dec_src_event (GstPad * pad, GstEvent * event)
 
       gst_event_parse_seek (event, &rate, &format, &flags, &cur_type, &cur,
           &stop_type, &stop);
+      gst_event_unref (event);
 
       /* we have to ask our peer to seek to time here as we know
        * nothing about how to generate a granulepos from the src
@@ -517,7 +537,8 @@ theora_dec_src_event (GstPad * pad, GstEvent * event)
 
       /* we cannot randomly skip frame decoding since we don't have
        * B frames. we can however use the timestamp and diff to not
-       * push late frames. */
+       * push late frames. This would let us save the time for copying and
+       * cropping the frame. */
       GST_OBJECT_LOCK (dec);
       dec->proportion = proportion;
       dec->earliest_time = timestamp + diff;
@@ -539,7 +560,6 @@ done:
 convert_error:
   {
     GST_DEBUG_OBJECT (dec, "could not convert format");
-    gst_event_unref (event);
     goto done;
   }
 }
@@ -550,7 +570,7 @@ theora_dec_sink_event (GstPad * pad, GstEvent * event)
   gboolean ret = FALSE;
   GstTheoraExpDec *dec;
 
-  dec = GST_THEORA_DEC (gst_pad_get_parent (pad));
+  dec = GST_THEORA_DEC_EXP (gst_pad_get_parent (pad));
 
   GST_LOG_OBJECT (dec, "handling event");
   switch (GST_EVENT_TYPE (event)) {
@@ -569,11 +589,11 @@ theora_dec_sink_event (GstPad * pad, GstEvent * event)
     {
       gboolean update;
       GstFormat format;
-      gdouble rate;
+      gdouble rate, arate;
       gint64 start, stop, time;
 
-      gst_event_parse_new_segment (event, &update, &rate, &format, &start,
-          &stop, &time);
+      gst_event_parse_new_segment_full (event, &update, &rate, &arate, &format,
+          &start, &stop, &time);
 
       /* we need TIME and a positive rate */
       if (format != GST_FORMAT_TIME)
@@ -583,8 +603,8 @@ theora_dec_sink_event (GstPad * pad, GstEvent * event)
         goto newseg_wrong_rate;
 
       /* now configure the values */
-      gst_segment_set_newsegment (&dec->segment, update,
-          rate, format, start, stop, time);
+      gst_segment_set_newsegment_full (&dec->segment, update,
+          rate, arate, format, start, stop, time);
 
       /* and forward */
       ret = gst_pad_push_event (dec->srcpad, event);
@@ -603,19 +623,17 @@ done:
 newseg_wrong_format:
   {
     GST_DEBUG_OBJECT (dec, "received non TIME newsegment");
+    gst_event_unref (event);
     goto done;
   }
 newseg_wrong_rate:
   {
     GST_DEBUG_OBJECT (dec, "negative rates not supported yet");
+    gst_event_unref (event);
     goto done;
   }
 }
 
-#define ROUND_UP_2(x) (((x) + 1) & ~1)
-#define ROUND_UP_4(x) (((x) + 3) & ~3)
-#define ROUND_UP_8(x) (((x) + 7) & ~7)
-
 static GstFlowReturn
 theora_handle_comment_packet (GstTheoraExpDec * dec, ogg_packet * packet)
 {
@@ -653,7 +671,7 @@ theora_handle_comment_packet (GstTheoraExpDec * dec, ogg_packet * packet)
         GST_TAG_NOMINAL_BITRATE, dec->info.target_bitrate, NULL);
   }
 
-  gst_element_found_tags_for_pad (GST_ELEMENT (dec), dec->srcpad, list);
+  gst_element_found_tags_for_pad (GST_ELEMENT_CAST (dec), dec->srcpad, list);
 
   return GST_FLOW_OK;
 }
@@ -691,8 +709,8 @@ theora_handle_type_packet (GstTheoraExpDec * dec, ogg_packet * packet)
 
   /* add black borders to make width/height/offsets even. we need this because
    * we cannot express an offset to the peer plugin. */
-  dec->width = ROUND_UP_2 (dec->info.pic_width + (dec->info.pic_x & 1));
-  dec->height = ROUND_UP_2 (dec->info.pic_height + (dec->info.pic_y & 1));
+  dec->width = GST_ROUND_UP_2 (dec->info.pic_width + (dec->info.pic_x & 1));
+  dec->height = GST_ROUND_UP_2 (dec->info.pic_height + (dec->info.pic_y & 1));
   dec->offset_x = dec->info.pic_x & ~1;
   dec->offset_y = dec->info.pic_y & ~1;
 
@@ -714,11 +732,14 @@ theora_handle_type_packet (GstTheoraExpDec * dec, ogg_packet * packet)
      * So, we convert to a widely-supported packed format.
      */
     fourcc = GST_MAKE_FOURCC ('Y', 'U', 'Y', '2');
-  } else {
-    /* TODO: Implement 4:4:4, check for reserved/invalid values,
-     * post appropriate error message, ensure callers handle errors here 
-     * properly.
+  } else if (dec->info.pixel_fmt == TH_PF_444) {
+    dec->output_bpp = 24;
+    /* As for I420, we can't define the stride for this, so we need to memcpy,
+     * though at least this is a planar format...
      */
+    fourcc = GST_MAKE_FOURCC ('Y', '4', '4', '4');
+  } else {
+    GST_ERROR_OBJECT (dec, "Invalid pixel format %d", dec->info.pixel_fmt);
     return GST_FLOW_ERROR;
   }
 
@@ -753,6 +774,7 @@ theora_handle_header_packet (GstTheoraExpDec * dec, ogg_packet * packet)
   if (ret < 0)
     goto header_read_error;
 
+  /* We can never get here unless we have at least a one-byte packet */
   switch (packet->packet[0]) {
     case 0x81:
       res = theora_handle_comment_packet (dec, packet);
@@ -780,11 +802,55 @@ header_read_error:
   }
 }
 
+/* returns TRUE if buffer is within segment, else FALSE.
+ * if buffer is on segment border, its timestamp and duration will be clipped */
+static gboolean
+clip_buffer (GstTheoraExpDec * dec, GstBuffer * buf)
+{
+  gboolean res = TRUE;
+  GstClockTime in_ts, in_dur, stop;
+  gint64 cstart, cstop;
+
+  in_ts = GST_BUFFER_TIMESTAMP (buf);
+  in_dur = GST_BUFFER_DURATION (buf);
+
+  GST_LOG_OBJECT (dec,
+      "timestamp:%" GST_TIME_FORMAT " , duration:%" GST_TIME_FORMAT,
+      GST_TIME_ARGS (in_ts), GST_TIME_ARGS (in_dur));
+
+  /* can't clip without TIME segment */
+  if (dec->segment.format != GST_FORMAT_TIME)
+    goto beach;
+
+  /* we need a start time */
+  if (!GST_CLOCK_TIME_IS_VALID (in_ts))
+    goto beach;
+
+  /* generate valid stop, if duration unknown, we have unknown stop */
+  stop =
+      GST_CLOCK_TIME_IS_VALID (in_dur) ? (in_ts + in_dur) : GST_CLOCK_TIME_NONE;
+
+  /* now clip */
+  if (!(res = gst_segment_clip (&dec->segment, GST_FORMAT_TIME,
+              in_ts, stop, &cstart, &cstop)))
+    goto beach;
+
+  /* update timestamp and possibly duration if the clipped stop time is valid */
+  GST_BUFFER_TIMESTAMP (buf) = cstart;
+  if (GST_CLOCK_TIME_IS_VALID (cstop))
+    GST_BUFFER_DURATION (buf) = cstop - cstart;
+
+beach:
+  GST_LOG_OBJECT (dec, "%sdropping", (res ? "not " : ""));
+  return res;
+}
+
+
 /* FIXME, this needs to be moved to the demuxer */
 static GstFlowReturn
 theora_dec_push (GstTheoraExpDec * dec, GstBuffer * buf)
 {
-  GstFlowReturn result;
+  GstFlowReturn result = GST_FLOW_OK;
   GstClockTime outtime = GST_BUFFER_TIMESTAMP (buf);
 
   if (outtime == GST_CLOCK_TIME_NONE) {
@@ -809,14 +875,32 @@ theora_dec_push (GstTheoraExpDec * dec, GstBuffer * buf)
 
         GST_DEBUG_OBJECT (dec, "patch buffer %lld %lld", size, time);
         GST_BUFFER_TIMESTAMP (buffer) = time;
+
+        if (dec->discont) {
+          GST_BUFFER_FLAG_SET (buffer, GST_BUFFER_FLAG_DISCONT);
+          dec->discont = FALSE;
+        }
+
         /* ignore the result.. */
-        gst_pad_push (dec->srcpad, buffer);
+        if (clip_buffer (dec, buffer))
+          gst_pad_push (dec->srcpad, buffer);
+        else
+          gst_buffer_unref (buffer);
         size--;
       }
       g_list_free (dec->queued);
       dec->queued = NULL;
     }
-    result = gst_pad_push (dec->srcpad, buf);
+
+    if (dec->discont) {
+      GST_BUFFER_FLAG_SET (buf, GST_BUFFER_FLAG_DISCONT);
+      dec->discont = FALSE;
+    }
+
+    if (clip_buffer (dec, buf))
+      result = gst_pad_push (dec->srcpad, buf);
+    else
+      gst_buffer_unref (buf);
   }
 
   return result;
@@ -838,7 +922,7 @@ theora_handle_422_image (GstTheoraExpDec * dec, th_ycbcr_buffer yuv,
   width = dec->width;
   height = dec->height;
 
-  stride = ROUND_UP_2 (width) * 2;
+  stride = GST_ROUND_UP_2 (width) * 2;
 
   out_size = stride * height;
 
@@ -855,7 +939,7 @@ theora_handle_422_image (GstTheoraExpDec * dec, th_ycbcr_buffer yuv,
    *
    * Do the interleaving... Note that this is kinda messed up if our width is
    * odd. In that case, we can't represent it properly in YUY2, so we just
-   * pad out to even in that case (this is why we have ROUND_UP_2() above).
+   * pad out to even in that case (this is why we have GST_ROUND_UP_2() above).
    */
   {
     guchar *src_y;
@@ -895,12 +979,81 @@ theora_handle_422_image (GstTheoraExpDec * dec, th_ycbcr_buffer yuv,
         *curdest = *src++;
         curdest += 4;
       }
-      src_cr += yuv[1].ystride;
+      src_cr += yuv[2].ystride;
 
       dest += stride;
     }
   }
 
+  GST_BUFFER_OFFSET (out) = dec->frame_nr;
+  if (dec->frame_nr != -1)
+    dec->frame_nr++;
+  GST_BUFFER_OFFSET_END (out) = dec->frame_nr;
+  GST_BUFFER_DURATION (out) =
+      gst_util_uint64_scale_int (GST_SECOND, dec->info.fps_denominator,
+      dec->info.fps_numerator);
+  GST_BUFFER_TIMESTAMP (out) = outtime;
+
+  return theora_dec_push (dec, out);
+
+no_buffer:
+  {
+    GST_DEBUG_OBJECT (dec, "could not get buffer, reason: %s",
+        gst_flow_get_name (result));
+    return result;
+  }
+}
+
+/* Get buffer, populate with original data (we must memcpy to get things to
+ * have the expected strides, etc...), and push.
+ */
+static GstFlowReturn
+theora_handle_444_image (GstTheoraExpDec * dec, th_ycbcr_buffer yuv,
+    GstClockTime outtime)
+{
+  int i, plane;
+  gint width, height;
+  gint out_size;
+  gint stride;
+  GstBuffer *out;
+  GstFlowReturn result;
+
+  width = dec->width;
+  height = dec->height;
+
+  /* TODO: Check if we have any special alignment requirements for the planes,
+   * or for each line within a plane. */
+  stride = width;
+  out_size = stride * height * 3;
+
+  /* now copy over the area contained in offset_x,offset_y,
+   * frame_width, frame_height */
+  result =
+      gst_pad_alloc_buffer_and_set_caps (dec->srcpad, GST_BUFFER_OFFSET_NONE,
+      out_size, GST_PAD_CAPS (dec->srcpad), &out);
+  if (result != GST_FLOW_OK)
+    goto no_buffer;
+
+  {
+    guchar *dest, *src;
+
+    for (plane = 0; plane < 3; plane++) {
+      /* TODO: do we have to use something different here? */
+      dest = GST_BUFFER_DATA (out) + plane * stride * height;
+
+      src = yuv[plane].data + dec->offset_x +
+          dec->offset_y * yuv[plane].ystride;
+
+      for (i = 0; i < height; i++) {
+        memcpy (dest, src, width);
+
+        dest += stride;
+        src += yuv[plane].ystride;
+      }
+
+    }
+  }
+
   /* FIXME, frame_nr not correct */
   GST_BUFFER_OFFSET (out) = dec->frame_nr;
   dec->frame_nr++;
@@ -942,8 +1095,8 @@ theora_handle_420_image (GstTheoraExpDec * dec, th_ycbcr_buffer yuv,
   /* should get the stride from the caps, for now we round up to the nearest
    * multiple of 4 because some element needs it. chroma needs special 
    * treatment, see videotestsrc. */
-  stride_y = ROUND_UP_4 (width);
-  stride_uv = ROUND_UP_8 (width) / 2;
+  stride_y = GST_ROUND_UP_4 (width);
+  stride_uv = GST_ROUND_UP_8 (width) / 2;
 
   out_size = stride_y * height + stride_uv * cheight * 2;
 
@@ -966,7 +1119,7 @@ theora_handle_420_image (GstTheoraExpDec * dec, th_ycbcr_buffer yuv,
     guchar *dest_y, *src_y;
     guchar *dest_u, *src_u;
     guchar *dest_v, *src_v;
-    guint offset_u, offset_v;
+    gint offset_u, offset_v;
 
     dest_y = GST_BUFFER_DATA (out);
     dest_u = dest_y + stride_y * height;
@@ -988,9 +1141,6 @@ theora_handle_420_image (GstTheoraExpDec * dec, th_ycbcr_buffer yuv,
     src_v = yuv[2].data + offset_v;
 
     for (i = 0; i < cheight; i++) {
-      /* TODO: This, like many other things, is broken for pixel formats other
-       * than OC_PF_420
-       */
       memcpy (dest_u, src_u, cwidth);
       memcpy (dest_v, src_v, cwidth);
 
@@ -1029,28 +1179,33 @@ theora_handle_data_packet (GstTheoraExpDec * dec, ogg_packet * packet,
   GstFlowReturn result;
   ogg_int64_t gp;
 
-  if (!dec->have_header)
+  if (G_UNLIKELY (!dec->have_header))
     goto not_initialized;
 
   if (th_packet_iskeyframe (packet)) {
     dec->need_keyframe = FALSE;
-  } else if (dec->need_keyframe) {
+  } else if (G_UNLIKELY (dec->need_keyframe)) {
     goto dropping;
   }
 
   /* this does the decoding */
-  if (th_decode_packetin (dec->dec, packet, &gp))
+  if (G_UNLIKELY (th_decode_packetin (dec->dec, packet, &gp)))
     goto decode_error;
 
   if (outtime != -1) {
     gboolean need_skip;
+    GstClockTime qostime;
+
+    /* QoS is done on running time */
+    qostime = gst_segment_to_running_time (&dec->segment, GST_FORMAT_TIME,
+        outtime);
 
     GST_OBJECT_LOCK (dec);
     /* check for QoS, don't perform the last steps of getting and
      * pushing the buffers that are known to be late. */
     /* FIXME, we can also entirely skip decoding if the next valid buffer is 
      * known to be after a keyframe (using the granule_shift) */
-    need_skip = dec->earliest_time != -1 && outtime <= dec->earliest_time;
+    need_skip = dec->earliest_time != -1 && qostime <= dec->earliest_time;
     GST_OBJECT_UNLOCK (dec);
 
     if (need_skip)
@@ -1059,20 +1214,21 @@ theora_handle_data_packet (GstTheoraExpDec * dec, ogg_packet * packet,
 
   /* this does postprocessing and set up the decoded frame
    * pointers in our yuv variable */
-  if (th_decode_ycbcr_out (dec->dec, yuv) < 0)
+  if (G_UNLIKELY (th_decode_ycbcr_out (dec->dec, yuv) < 0))
     goto no_yuv;
 
-  if ((yuv[0].width != dec->info.frame_width) ||
-      (yuv[0].height != dec->info.frame_height))
+  if (G_UNLIKELY ((yuv[0].width != dec->info.frame_width) ||
+          (yuv[0].height != dec->info.frame_height)))
     goto wrong_dimensions;
 
   if (dec->info.pixel_fmt == TH_PF_420) {
     result = theora_handle_420_image (dec, yuv, outtime);
   } else if (dec->info.pixel_fmt == TH_PF_422) {
     result = theora_handle_422_image (dec, yuv, outtime);
+  } else if (dec->info.pixel_fmt == TH_PF_444) {
+    result = theora_handle_444_image (dec, yuv, outtime);
   } else {
-    /* Should be unreachable */
-    result = GST_FLOW_ERROR;
+    g_assert_not_reached ();
   }
 
   return result;
@@ -1087,11 +1243,14 @@ not_initialized:
 dropping:
   {
     GST_WARNING_OBJECT (dec, "dropping frame because we need a keyframe");
+    dec->discont = TRUE;
     return GST_FLOW_OK;
   }
 dropping_qos:
   {
-    dec->frame_nr++;
+    if (dec->frame_nr != -1)
+      dec->frame_nr++;
+    dec->discont = TRUE;
     GST_WARNING_OBJECT (dec, "dropping frame because of QoS");
     return GST_FLOW_OK;
   }
@@ -1121,14 +1280,17 @@ theora_dec_chain (GstPad * pad, GstBuffer * buf)
   GstTheoraExpDec *dec;
   ogg_packet packet;
   GstFlowReturn result = GST_FLOW_OK;
+  gboolean isheader;
 
-  dec = GST_THEORA_DEC (gst_pad_get_parent (pad));
+  dec = GST_THEORA_DEC_EXP (gst_pad_get_parent (pad));
 
   /* resync on DISCONT */
   if (G_UNLIKELY (GST_BUFFER_FLAG_IS_SET (buf, GST_BUFFER_FLAG_DISCONT))) {
+    GST_DEBUG_OBJECT (dec, "Received DISCONT buffer");
     dec->need_keyframe = TRUE;
     dec->last_timestamp = -1;
     dec->granulepos = -1;
+    dec->discont = TRUE;
   }
 
   GST_DEBUG ("Offset end is %d, k-g-s %d", (int) (GST_BUFFER_OFFSET_END (buf)),
@@ -1156,16 +1318,27 @@ theora_dec_chain (GstPad * pad, GstBuffer * buf)
       GST_DEBUG_OBJECT (dec, "Granulepos unknown");
       dec->last_timestamp = GST_CLOCK_TIME_NONE;
     }
+    if (dec->last_timestamp == GST_CLOCK_TIME_NONE &&
+        GST_BUFFER_TIMESTAMP_IS_VALID (buf))
+      dec->last_timestamp = GST_BUFFER_TIMESTAMP (buf);
+
   } else {
     GST_DEBUG_OBJECT (dec, "Granulepos not usable: no headers seen");
     dec->last_timestamp = -1;
   }
 
+  /* A zero-byte packet is a valid data packet, meaning 'duplicate frame' */
+  if (packet.bytes > 0 && packet.packet[0] & 0x80)
+    isheader = TRUE;
+  else
+    isheader = FALSE;
+
   GST_DEBUG_OBJECT (dec, "header=%d packetno=%lld, outtime=%" GST_TIME_FORMAT,
-      packet.packet[0], packet.packetno, GST_TIME_ARGS (dec->last_timestamp));
+      packet.bytes ? packet.packet[0] : -1, packet.packetno,
+      GST_TIME_ARGS (dec->last_timestamp));
 
   /* switch depending on packet type */
-  if (packet.packet[0] & 0x80) {
+  if (isheader) {
     if (dec->have_header) {
       GST_WARNING_OBJECT (GST_OBJECT (dec), "Ignoring header");
       goto done;
@@ -1189,7 +1362,7 @@ done:
 static GstStateChangeReturn
 theora_dec_change_state (GstElement * element, GstStateChange transition)
 {
-  GstTheoraExpDec *dec = GST_THEORA_DEC (element);
+  GstTheoraExpDec *dec = GST_THEORA_DEC_EXP (element);
   GstStateChangeReturn ret;
 
 
@@ -1219,6 +1392,7 @@ theora_dec_change_state (GstElement * element, GstStateChange transition)
 
       th_comment_clear (&dec->comment);
       th_info_clear (&dec->info);
+      gst_theoradec_reset (dec);
       break;
     case GST_STATE_CHANGE_READY_TO_NULL:
       break;
