@@ -11,7 +11,6 @@
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-
  * Library General Public License for more details.
  *
  * You should have received a copy of the GNU Library General Public
@@ -20,6 +19,29 @@
  * Boston, MA 02111-1307, USA.
  */
 
+/**
+ * SECTION:element-wildmidi
+ * @see_also: timidity
+ *
+ * <refsect2>
+ * <para>
+ * This element renders midi-files as audio streams using
+ * <ulink url="http://wildmidi.sourceforge.net//">Wildmidi</ulink>.
+ * It offers better sound quality compared to the timidity element. Wildmidi
+ * uses the same sound-patches as timidity (it tries the path in $WILDMIDI_CFG,
+ * $HOME/.wildmidirc and /etc/wildmidi.cfg)
+ * </para>
+ * <para>
+ * </para>
+ * <title>Example pipeline</title>
+ * <programlisting>
+ * gst-launch filesrc location=song.mid ! wildmidi ! alsasink
+ * </programlisting>
+ * This example pipeline will parse the midi and render to raw audio which is
+ * played via alsa.
+ * </refsect2>
+ */
+
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
 #endif
@@ -31,6 +53,7 @@
 #include <string.h>
 #include <glib.h>
 #include <glib/gstdio.h>
+#include <stdlib.h>
 #include <unistd.h>
 
 #include "gstwildmidi.h"
@@ -103,6 +126,75 @@ gst_wildmidi_base_init (gpointer gclass)
   gst_element_class_set_details (element_class, &gst_wildmidi_details);
 }
 
+static gboolean
+wildmidi_open_config ()
+{
+  gchar *path = g_strdup (g_getenv ("WILDMIDI_CFG"));
+  gint ret;
+
+  GST_DEBUG ("trying %s", GST_STR_NULL (path));
+  if (path && (g_access (path, R_OK) == -1)) {
+    g_free (path);
+    path = NULL;
+  }
+
+  if (path == NULL) {
+    path =
+        g_build_path (G_DIR_SEPARATOR_S, g_get_home_dir (), ".wildmidirc",
+        NULL);
+    GST_DEBUG ("trying %s", path);
+    if (path && (g_access (path, R_OK) == -1)) {
+      g_free (path);
+      path = NULL;
+    }
+  }
+
+  if (path == NULL) {
+    path = g_build_path (G_DIR_SEPARATOR_S, "/etc", "wildmidi.cfg", NULL);
+    GST_DEBUG ("trying %s", path);
+    if (path && (g_access (path, R_OK) == -1)) {
+      g_free (path);
+      path = NULL;
+    }
+  }
+
+  if (path == NULL) {
+    path = g_strdup (TIMIDITY_CFG);
+    GST_DEBUG ("trying %s", path);
+    if (path && (g_access (path, R_OK) == -1)) {
+      g_free (path);
+      path = NULL;
+    }
+  }
+
+  if (path == NULL) {
+    path = g_build_path (G_DIR_SEPARATOR_S, "/etc", "timidity.cfg", NULL);
+    GST_DEBUG ("trying %s", path);
+    if (path && (g_access (path, R_OK) == -1)) {
+      g_free (path);
+      path = NULL;
+    }
+  }
+
+  if (path == NULL) {
+    /* I've created a symlink to get it playing
+     * ln -s /usr/share/timidity/timidity.cfg /etc/wildmidi.cfg
+     * we could make it use : TIMIDITY_CFG
+     * but unfortunately it fails to create a proper filename if the config
+     * has a redirect   
+     * http://sourceforge.net/tracker/index.php?func=detail&aid=1657358&group_id=42635&atid=433744
+     */
+    GST_WARNING ("no config file, can't initialise");
+    return FALSE;
+  }
+
+  /* this also initializes a some filter and stuff and thus is slow */
+  ret = WildMidi_Init (path, WILDMIDI_RATE, 0);
+  g_free (path);
+
+  return (ret == 0);
+}
+
 /* initialize the plugin's class */
 static void
 gst_wildmidi_class_init (GstWildmidiClass * klass)
@@ -136,6 +228,13 @@ gst_wildmidi_init (GstWildmidi * filter, GstWildmidiClass * g_class)
 {
   GstElementClass *klass = GST_ELEMENT_GET_CLASS (filter);
 
+  /* initialise wildmidi library */
+  if (wildmidi_open_config ()) {
+    filter->initialized = TRUE;
+  } else {
+    GST_WARNING ("can't initialize wildmidi");
+  }
+
   filter->sinkpad =
       gst_pad_new_from_template (gst_element_class_get_pad_template (klass,
           "sink"), "sink");
@@ -699,6 +798,11 @@ gst_wildmidi_change_state (GstElement * element, GstStateChange transition)
   GstStateChangeReturn ret = GST_STATE_CHANGE_SUCCESS;
   GstWildmidi *wildmidi = GST_WILDMIDI (element);
 
+  if (!wildmidi->initialized) {
+    GST_WARNING ("WildMidi renderer is not initialized");
+    return GST_STATE_CHANGE_FAILURE;
+  }
+
   switch (transition) {
     case GST_STATE_CHANGE_NULL_TO_READY:
       wildmidi->out_caps =
@@ -804,77 +908,23 @@ gst_wildmidi_typefind (GstTypeFind * tf, gpointer _data)
   }
 }
 
-static gboolean
-wildmidi_open_config ()
-{
-  gchar *path = g_strdup (g_getenv ("WILDMIDI_CFG"));
-  gint ret;
-
-  GST_DEBUG ("trying %s", path);
-  if (path && (g_access (path, R_OK) == -1)) {
-    g_free (path);
-    path = NULL;
-  }
-
-  if (path == NULL) {
-    path =
-        g_build_path (G_DIR_SEPARATOR_S, g_get_home_dir (), ".wildmidirc",
-        NULL);
-    GST_DEBUG ("trying %s", path);
-    if (path && (g_access (path, R_OK) == -1)) {
-      g_free (path);
-      path = NULL;
-    }
-  }
-
-  if (path == NULL) {
-    path = g_build_path (G_DIR_SEPARATOR_S, "/etc", "wildmidi.cfg", NULL);
-    GST_DEBUG ("trying %s", path);
-    if (path && (g_access (path, R_OK) == -1)) {
-      g_free (path);
-      path = NULL;
-    }
-  }
-
-  if (path == NULL) {
-    /* I've created a symlink to get it playing
-     * ln -s /usr/share/timidity/timidity.cfg /etc/wildmidi.cfg
-     * we could make it use : TIMIDITY_CFG
-     * but unfortunately it fails to create a proper filename if the config
-     * has a redirect   
-     * http://sourceforge.net/tracker/index.php?func=detail&aid=1657358&group_id=42635&atid=433744
-     */
-    GST_INFO ("no config file");
-    return FALSE;
-  }
-
-  ret = WildMidi_Init (path, WILDMIDI_RATE, 0);
-  g_free (path);
-
-  return (ret == 0);
-}
-
 static gboolean
 plugin_init (GstPlugin * plugin)
 {
   static gchar *exts[] = { "mid", "midi", NULL };
-  /* exchange the strings 'plugin' and 'Template plugin' with your
-   * plugin name and description */
+  GstCaps *caps;
+
   GST_DEBUG_CATEGORY_INIT (gst_wildmidi_debug, "wildmidi",
       0, "Wildmidi plugin");
 
-  /* initialise wildmidi library, fail loading the plugin if this fails */
-  if (!wildmidi_open_config ()) {
-    GST_WARNING ("can't initialize wildmidi");
-    return FALSE;
-  }
-
+  caps = gst_caps_new_simple ("audio/midi", NULL);
   if (!gst_type_find_register (plugin, "audio/midi", GST_RANK_SECONDARY,
-          gst_wildmidi_typefind, exts,
-          gst_caps_new_simple ("audio/midi", NULL), NULL, NULL)) {
+          gst_wildmidi_typefind, exts, caps, NULL, NULL)) {
     GST_WARNING ("can't register typefind");
+    gst_caps_unref (caps);
     return FALSE;
   }
+  gst_caps_unref (caps);
 
   return gst_element_register (plugin, "wildmidi",
       GST_RANK_SECONDARY, GST_TYPE_WILDMIDI);
@@ -884,4 +934,4 @@ GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
     GST_VERSION_MINOR,
     "wildmidi",
     "Wildmidi Plugin",
-    plugin_init, VERSION, "LGPL", "GStreamer", "http://gstreamer.net/")
+    plugin_init, VERSION, "GPL", "GStreamer", "http://gstreamer.net/")
